@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22195 $
+ * $Revision: 24410 $
  * $Author: rjongbloed $
- * $Date: 2009-03-11 19:33:55 -0500 (Wed, 11 Mar 2009) $
+ * $Date: 2010-05-27 18:43:58 -0500 (Thu, 27 May 2010) $
  */
 
 #ifndef PTLIB_PSOCKBUN_H
@@ -166,10 +166,9 @@ class PInterfaceMonitor : public PProcessStartup
     PTimeInterval  m_refreshInterval;
     PMutex         m_threadMutex;
     PThread      * m_updateThread;
-    PSyncPoint     m_signalUpdate;
-    bool           m_threadRunning;
-    
+
     PInterfaceFilter * m_interfaceFilter;
+    PIPSocket::RouteTableDetector * m_changedDetector;
 
   friend class PInterfaceMonitorClient;
 };
@@ -623,7 +622,7 @@ class PSingleMonitoredSocket : public PMonitoredSockets
         ip%name, eg "10.0.1.11%3Com 3C90x Ethernet Adapter" or "192.168.0.10%eth0"
       */
     virtual PStringArray GetInterfaces(
-      PBoolean includeLoopBack = false,  /// Flag for if loopback is to included in list
+      bool includeLoopBack = false,  /// Flag for if loopback is to included in list
       const PIPSocket::Address & destination = PIPSocket::GetDefaultIpAny()
     );
 
