@@ -26,15 +26,17 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 23005 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Date: 2009-06-28 23:07:13 -0500 (Sun, 28 Jun 2009) $
  */
 
 ///////////////////////////////////////////////////////////////////////////////
 // PConsoleChannel
 
   public:
+    ~PConsoleChannel();
+
     // Overrides from class PChannel
     virtual PString GetName() const;
       // Return the name of the channel.
@@ -51,4 +53,7 @@
     virtual PBoolean Close();
       // Close the channel.
 
+  protected:
+    HANDLE m_hConsole;
+
 // End Of File ///////////////////////////////////////////////////////////////
