@@ -5,9 +5,9 @@
  *
  * Portable Windows Library
  *
- * $Revision: 20385 $
+ * $Revision: 24473 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Date: 2010-06-07 19:39:21 -0500 (Mon, 07 Jun 2010) $
  */
 
 ///////////////////////////////////////////////////////////////////////
@@ -766,13 +766,16 @@ PBoolean PASN_Sequence::PreambleDecodePER(PPER_Stream & strm)
 {
   // X.691 Section 18
 
+  totalExtensions = 0;
+  extensionMap.SetSize(0);
+
   if (extendable) {
     if (strm.IsAtEnd())
       return PFalse;
-    totalExtensions = strm.SingleBitDecode() ? -1 : 0;  // 18.1
+    if (strm.SingleBitDecode()) // 18.1
+      totalExtensions = -1;
   }
-  else
-    totalExtensions = 0;
+
   return optionMap.Decode(strm);  // 18.2
 }
 
