@@ -27,9 +27,9 @@
  * Original copyright notice appears below
  *
  * $Id: pssl.cxx 19752 2008-03-15 15:30:24Z hfriederich $
- * $Revision: 23262 $
+ * $Revision: 23261 $
  * $Author: rjongbloed $
- * $Date: 2009-08-24 19:25:25 -0500 (Mon, 24 Aug 2009) $
+ * $Date: 2009-08-24 19:09:09 -0500 (Mon, 24 Aug 2009) $
  */
 
 /* crypto/buffer/bss_sock.c */
@@ -658,7 +658,7 @@ PSSLDiffieHellman::~PSSLDiffieHellman()
     DH_free(dh);
 }
 
-#if defined(__BEOS__) || (defined(__APPLE__) && P_MACOSX < 900) || defined(__OpenBSD__)
+#ifdef P_d2i_DHparams_bio_OLD
 // 2/21/04 Yuri Kiryanov - fix for compiler choke on BeOS for usage of
 // SSL function d2i_DHparams_bio below in PSSLDiffieHellman::Load
 // 5/26/06 Hannes Friederich - Mac OS X seems to need that fix too...
