@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 23589 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Date: 2009-10-05 19:11:03 -0500 (Mon, 05 Oct 2009) $
  */
 
 #ifdef __GNUC__
@@ -38,7 +38,6 @@
 #include <ptlib/pfactory.h>
 #include <ptclib/ptts.h>
 
-PINSTANTIATE_FACTORY(PTextToSpeech, PString)
 
 // WIN32 COM stuff must be first in file to compile properly
 
@@ -119,7 +118,7 @@ class PTextToSpeech_SAPI : public PTextToSpeech
     PString voice;
 };
 
-PFactory<PTextToSpeech>::Worker<PTextToSpeech_SAPI> sapiTTSFactory("Microsoft SAPI", false);
+PFACTORY_CREATE(PTextToSpeech, PTextToSpeech_SAPI, "Microsoft SAPI", false);
 
 int * PTextToSpeech_SAPI::refCount;
 PMutex PTextToSpeech_SAPI::refMutex;
@@ -322,6 +321,9 @@ unsigned PTextToSpeech_SAPI::GetVolume()
 #endif
 // P_SAPI
 
+
+#ifndef _WIN32_WCE
+
 ////////////////////////////////////////////////////////////
 //
 //  Generic text to speech using Festival
@@ -367,7 +369,7 @@ class PTextToSpeech_Festival : public PTextToSpeech
 
 #define new PNEW
 
-PFactory<PTextToSpeech>::Worker<PTextToSpeech_Festival> festivalTTSFactory("Festival", false);
+PFACTORY_CREATE(PFactory<PTextToSpeech>, PTextToSpeech_Festival, "Festival", false);
 
 PTextToSpeech_Festival::PTextToSpeech_Festival()
 {
@@ -510,11 +512,7 @@ PBoolean PTextToSpeech_Festival::Invoke(const PString & otext, const PFilePath &
 
 #if 1
 
-#ifndef _WIN32_WCE
-  system(cmdLine);
-#endif
-
-  return PTrue;
+  return system(cmdLine) != -1;
 
 #else
 
@@ -538,4 +536,8 @@ PBoolean PTextToSpeech_Festival::Invoke(const PString & otext, const PFilePath &
 #endif
 }
 
+
+#endif
+
+
 // End Of File ///////////////////////////////////////////////////////////////
