@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 23711 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Date: 2009-10-28 03:45:13 -0500 (Wed, 28 Oct 2009) $
  */
 
 // This depends on the expat XML library by Jim Clark
@@ -54,7 +54,7 @@ static const char NoIndentElements[] = "methodName name string int boolean doubl
 /////////////////////////////////////////////////////////////////
 
 PXMLRPCBlock::PXMLRPCBlock()
-  : PXML(-1, NoIndentElements)
+  : PXML(PXMLParser::NoOptions, NoIndentElements)
 {
   faultCode = P_MAX_INDEX;
   SetRootElement("methodResponse");
@@ -62,7 +62,7 @@ PXMLRPCBlock::PXMLRPCBlock()
 }
 
 PXMLRPCBlock::PXMLRPCBlock(const PString & method)
-  : PXML(-1, NoIndentElements)
+  : PXML(PXMLParser::NoOptions, NoIndentElements)
 {
   faultCode = P_MAX_INDEX;
   SetRootElement("methodCall");
@@ -71,7 +71,7 @@ PXMLRPCBlock::PXMLRPCBlock(const PString & method)
 }
 
 PXMLRPCBlock::PXMLRPCBlock(const PString & method, const PXMLRPCStructBase & data)
-  : PXML(-1, NoIndentElements)
+  : PXML(PXMLParser::NoOptions, NoIndentElements)
 {
   faultCode = P_MAX_INDEX;
   SetRootElement("methodCall");
@@ -712,7 +712,8 @@ PXMLElement * PXMLRPCBlock::GetParam(PINDEX idx) const
 
   PXMLElement * param = NULL;
   PINDEX i;
-  for (i = 0; i < params->GetSize(); i++) {
+  PINDEX s = params->GetSize();
+  for (i = 0; i < s; i++) {
     PXMLElement * element = (PXMLElement *)params->GetElement(i);
     if (element != NULL && element->IsElement() && element->GetName() == "param") {
       if (idx <= 0) {
@@ -875,11 +876,11 @@ PBoolean PXMLRPCBlock::GetParam(PINDEX idx, PXMLRPCStructBase & data)
 
 ////////////////////////////////////////////////////////
 
-PXMLRPC::PXMLRPC(const PURL & _url, unsigned opts)
+PXMLRPC::PXMLRPC(const PURL & _url, PXMLParser::Options opts)
   : url(_url)
+  , timeout(0, 10) // Seconds
+  , m_options(opts)
 {
-  timeout = 10000;
-  options = opts;
 }
 
 PBoolean PXMLRPC::MakeRequest(const PString & method)
@@ -928,7 +929,7 @@ PBoolean PXMLRPC::PerformRequest(PXMLRPCBlock & request, PXMLRPCBlock & response
 {
   // create XML version of request
   PString requestXML;
-  if (!request.Save(requestXML, options)) {
+  if (!request.Save(requestXML, m_options)) {
     PStringStream txt;
     txt << "Error creating request XML ("
         << request.GetErrorLine() 
