@@ -26,18 +26,24 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21994 $
+ * $Revision: 24357 $
  * $Author: rjongbloed $
- * $Date: 2009-02-04 22:46:56 +0000 (Wed, 04 Feb 2009) $
+ * $Date: 2010-05-18 02:52:29 -0500 (Tue, 18 May 2010) $
  */
 
 #include <ptlib.h>
 
-#ifndef _WIN32_WCE
+
+PDEFINE_POOL_ALLOCATOR(PListElement)
+PDEFINE_POOL_ALLOCATOR(PListInfo)
+PDEFINE_POOL_ALLOCATOR(PSortedListElement)
+PDEFINE_POOL_ALLOCATOR(PSortedListInfo)
+PDEFINE_POOL_ALLOCATOR(PHashTableElement)
+
+
 #define new PNEW
 #undef  __CLASS__
 #define __CLASS__ GetClass()
-#endif
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -45,7 +51,7 @@
 void PCollection::PrintOn(ostream &strm) const
 {
   char separator = strm.fill();
-  int width = strm.width();
+  int width = (int)strm.width();
   for (PINDEX  i = 0; i < GetSize(); i++) {
     if (i > 0 && separator != ' ')
       strm << separator;
@@ -188,7 +194,9 @@ PBoolean PArrayObjects::SetAt(PINDEX index, PObject * obj)
 
 PINDEX PArrayObjects::InsertAt(PINDEX index, PObject * obj)
 {
-  for (PINDEX i = GetSize(); i > index; i--)
+  PINDEX i = GetSize();
+  SetSize(i+1);
+  for (; i > index; i--)
     (*theArray)[i] = (*theArray)[i-1];
   (*theArray)[index] = obj;
   return index;
