@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23595 $
+ * $Revision: 24188 $
  * $Author: rjongbloed $
- * $Date: 2009-10-06 17:57:33 -0500 (Tue, 06 Oct 2009) $
+ * $Date: 2010-04-06 03:19:20 -0500 (Tue, 06 Apr 2010) $
  */
 
 #include <ptlib.h>
@@ -41,47 +41,12 @@
 #include <ptlib/pprocess.h>
 #include <ptlib/svcproc.h>
 #include "../../../version.h"
+#include "../../../revision.h"
 
 #ifdef _WIN32
 #include <ptlib/msos/ptlib/debstrm.h>
 #endif
 
-namespace PWLibStupidLinkerHacks {
-
-
-#if defined(P_MINGW) || defined(_WIN32) || defined(P_MACOSX)
-
-#  ifdef P_VIDEO
-     extern int loadFakeVideoStuff;
-#    if defined(P_MINGW) || defined(_WIN32)
-#      ifdef P_VFW_CAPTURE
-         extern int loadVideoForWindowsStuff;
-#      endif
-#      if defined(P_DIRECTSHOW)
-         extern int loadDirectShowStuff;
-#      endif
-#    endif
-#    ifdef P_FFVDEV
-       extern int loadFFVideoFileStuff;
-#    endif
-#    ifdef P_VIDFILE
-       extern int loadVideoFileStuff;
-#    endif
-#  endif 
-
-#  ifdef P_AUDIO
-#    if defined(P_MINGW) || defined(_WIN32)
-       extern int loadWindowsMultimediaStuff;
-#    endif
-#    ifdef P_MACOSX
-       extern int loadCoreAudioStuff;
-#    endif
-#  endif
-
-#endif // defined(P_MINGW) || defined(_WIN32) || defined(P_MACOSX)
-
-}; // namespace PWLibStupidLinkerHacks {
-
 
 static const char * const VersionStatus[PProcess::NumCodeStatuses] = { "alpha", "beta", "." };
 
@@ -558,13 +523,17 @@ ostream & PTrace::End(ostream & paramStream)
 
   if (threadInfo != NULL) {
     PStringStream * stackStream = threadInfo->traceStreams.Pop();
-    PAssert(&paramStream == stackStream, PLogicError);
+    if (!PAssert(&paramStream == stackStream, PLogicError))
+      return paramStream;
+    *stackStream << ends << flush;
     info.Lock();
     *info.stream << *stackStream;
     delete stackStream;
   }
   else {
-    PAssert(&paramStream == info.stream, PLogicError);
+    if (!PAssert(&paramStream == info.stream, PLogicError))
+      return paramStream;
+    info.Lock();
   }
 
   if ((info.options&SystemLogStream) != 0) {
@@ -661,33 +630,26 @@ void PDirectory::CloneContents(const PDirectory * d)
 }
 
 
-///////////////////////////////////////////////////////////////////////////////
-// PTimeInterval
-
-DWORD PTimeInterval::GetInterval() const
-{
-  if (milliseconds <= 0)
-    return 0;
-
-  if (milliseconds >= UINT_MAX)
-    return UINT_MAX;
-
-  return (DWORD)milliseconds;
-}
-
-
 ///////////////////////////////////////////////////////////////////////////////
 // PTimer
 
+
 PTimer::PTimer(long millisecs, int seconds, int minutes, int hours, int days)
-  : resetTime(millisecs, seconds, minutes, hours, days)
+  : m_resetTime(millisecs, seconds, minutes, hours, days)
 {
   Construct();
 }
 
 
 PTimer::PTimer(const PTimeInterval & time)
-  : resetTime(time)
+  : m_resetTime(time)
+{
+  Construct();
+}
+
+
+PTimer::PTimer(const PTimer & timer)
+  : m_resetTime(timer.GetMilliSeconds())
 {
   Construct();
 }
@@ -695,34 +657,51 @@ PTimer::PTimer(const PTimeInterval & time)
 
 void PTimer::Construct()
 {
-  timerList = PProcess::Current().GetTimerList();
-  timerId = timerList->GetNewTimerId();
-  state = Stopped;
+  m_timerList = PProcess::Current().GetTimerList();
+  m_timerId = m_timerList->GetNewTimerId();
+  m_state = Stopped;
 
   StartRunning(PTrue);
 }
 
 
-PTimer & PTimer::operator=(DWORD milliseconds)
+PInt64 PTimer::GetMilliSeconds() const
 {
-  resetTime.SetInterval(milliseconds);
-  StartRunning(oneshot);
-  return *this;
+  PInt64 diff = m_absoluteTime - Tick().GetMilliSeconds();
+  if (diff < 0)
+    diff = 0;
+  return diff;
 }
 
 
+PTimer & PTimer::operator=(DWORD milliseconds)
+{
+  m_resetTime.SetInterval(milliseconds);
+  StartRunning(m_oneshot);
+  return *this;
+}
+ 
+
 PTimer & PTimer::operator=(const PTimeInterval & time)
 {
-  resetTime = time;
-  StartRunning(oneshot);
+  m_resetTime = time;
+  StartRunning(m_oneshot);
   return *this;
 }
 
 
+PTimer & PTimer::operator=(const PTimer & timer)
+{
+  m_resetTime.SetInterval(timer.GetMilliSeconds());
+  StartRunning(m_oneshot);
+  return *this;
+}
+ 
+ 
 PTimer::~PTimer()
 {
   // queue a request to remove this timer, and always do it synchronously
-  timerList->QueueRequest(PTimerList::RequestType::Stop, this, true);
+  Stop(true);
 }
 
 
@@ -732,37 +711,42 @@ void PTimer::SetInterval(PInt64 milliseconds,
                          long hours,
                          int days)
 {
-  resetTime.SetInterval(milliseconds, seconds, minutes, hours, days);
-  StartRunning(oneshot);
+  m_resetTime.SetInterval(milliseconds, seconds, minutes, hours, days);
+  StartRunning(m_oneshot);
 }
 
 
 void PTimer::RunContinuous(const PTimeInterval & time)
 {
-  resetTime = time;
+  m_resetTime = time;
   StartRunning(PFalse);
 }
 
 
 void PTimer::StartRunning(PBoolean once)
 {
-  PTimeInterval::operator=(resetTime);
-  oneshot = once;
-  state = (*this) != 0 ? Starting : Stopped;
+  PTimeInterval::operator=(m_resetTime);
+  m_oneshot = once;
+  int oldState = m_state;
+  m_state = (m_resetTime == 0 ? Stopped : Running);
 
-  if (IsRunning())
-    timerList->QueueRequest(PTimerList::RequestType::Start, this, false);
-  else if (state != Stopped)
-    timerList->QueueRequest(PTimerList::RequestType::Stop, this);
+  if (!IsRunning() && (oldState != Stopped)) 
+    m_timerList->QueueRequest(PTimerList::RequestType::Stop, this);
+  else if (IsRunning()) {
+    if (oldState != Stopped)
+      m_timerList->QueueRequest(PTimerList::RequestType::Stop, this, false);
+
+    m_absoluteTime = Tick().GetMilliSeconds() + m_resetTime.GetMilliSeconds();
+    m_timerList->QueueRequest(PTimerList::RequestType::Start, this, false);
+  }
 }
 
 
 void PTimer::Stop(bool wait)
 {
-  if (state != Stopped) {
-    state = Stopped;
-    milliseconds = 0;
-    timerList->QueueRequest(PTimerList::RequestType::Stop, this, wait);
+  if (m_state != Stopped) {
+    m_state = Stopped;
+    m_timerList->QueueRequest(PTimerList::RequestType::Stop, this, wait);
   }
 }
 
@@ -770,72 +754,42 @@ void PTimer::Stop(bool wait)
 void PTimer::Pause()
 {
   if (IsRunning()) {
-    state = Paused;
-    timerList->QueueRequest(PTimerList::RequestType::Stop, this);
+    m_state = Paused;
+    m_timerList->QueueRequest(PTimerList::RequestType::Stop, this);
   }
 }
 
 
 void PTimer::Resume()
 {
-  if (state == Paused) {
-    state = Starting;
-    timerList->QueueRequest(PTimerList::RequestType::Start, this);
+  if (m_state == Stopped || m_state == Paused) {
+    m_state = Running;
+    m_timerList->QueueRequest(PTimerList::RequestType::Start, this);
   }
 }
 
 
 void PTimer::Reset()
 {
-  StartRunning(oneshot);
+  StartRunning(m_oneshot);
 }
 
 // called only from the timer thread
 void PTimer::OnTimeout()
 {
-  if (!callback.IsNULL())
-    callback(*this, IsRunning());
+  if (!m_callback.IsNULL())
+    m_callback(*this, IsRunning());
 }
 
 
-void PTimer::Process(const PTimeInterval & delta, PTimeInterval & minTimeLeft)
+void PTimer::Process(PInt64 now)
 {
-  switch (state) {
-    case Starting :
-      state = Running;
-      if (resetTime < minTimeLeft)
-        minTimeLeft = resetTime;
-      break;
-
+  switch (m_state) {
     case Running :
-      operator-=(delta);
-
-      if (milliseconds > 0) {
-        if (milliseconds < minTimeLeft.GetMilliSeconds())
-          minTimeLeft = *this;
-      }
-      else {
-        if (oneshot) {
-          milliseconds = 0;
-          state = Stopped;
-        }
-        else {
-          PTimeInterval::operator=(resetTime);
-          if (resetTime < minTimeLeft)
-            minTimeLeft = resetTime;
-        }
-
-        /* This must be outside the mutex or if OnTimeout() changes the
-           timer value (quite plausible) it deadlocks.
-         */
+      if (m_absoluteTime <= now) {
+        if (m_oneshot) 
+          m_state = Stopped;
         OnTimeout();
-
-        if (state == Starting) {
-          state = Running;
-          if (resetTime < minTimeLeft)
-            minTimeLeft = resetTime;
-        }
-        return;
       }
       break;
 
@@ -850,145 +804,127 @@ void PTimer::Process(const PTimeInterval & delta, PTimeInterval & minTimeLeft)
 
 PTimerList::PTimerList()
 {
-  timerThread = NULL;
+  m_timerThread = NULL;
 }
 
-void PTimerList::QueueRequest(RequestType::Action action, PTimer * timer, bool _isSync)
+void PTimerList::QueueRequest(RequestType::Action action, PTimer * timer, bool isSync)
 {
-  // if this operation is occurring in the timer thread, then handle synchronoously
-  if (timerThread == PThread::Current()) {
-    switch (action) {
-      case PTimerList::RequestType::Start:
-        {
-          TimerInfoMapType::iterator r = activeTimers.find(timer->GetTimerId());
+  bool inTimerThread = m_timerThread == PThread::Current();
 
-          // if the timer is a new timer, then use special queue as asynchronous add won't work
-          if (r == activeTimers.end()) {
-            RequestType request(action, timer);
-            queueMutex.Wait();
-            addQueue.push(request);
-            queueMutex.Signal();
-          }
-        }
-        break;
-      case PTimerList::RequestType::Stop:
-        {
-          // flag the timer as to-be-removed
-          TimerInfoMapType::iterator r = activeTimers.find(timer->GetTimerId());
-          if (r != activeTimers.end())
-            r->second.removed = true;
-        }
-        break;
-    }
-    return;
-  }
-
-  // submit asynchronous request
   RequestType request(action, timer);
   PSyncPoint sync;
-  bool isSync = false;
-  if (_isSync) {
-    request.sync = &sync;
-    isSync = true;
-  }
 
-  queueMutex.Wait();
-  requestQueue.push(request);
-  queueMutex.Signal();
+  // set synchronisation point
+  if (!inTimerThread) 
+    request.m_sync = isSync ? &sync : NULL;
 
-  PProcess::Current().SignalTimerChange();
+  // queue the request
+  m_queueMutex.Wait();
+  m_requestQueue.push(request);
+  m_queueMutex.Signal();
 
-  if (isSync)
+  // wait for synchronisation point
+  if (!inTimerThread && PProcess::Current().SignalTimerChange() && isSync)
     sync.Wait();
 }
 
-PTimeInterval PTimerList::Process()
+
+void PTimerList::AddActiveTimer(const RequestType & request)
 {
-  timerThread = PThread::Current();
+  ActiveTimerInfoMap::iterator r = m_activeTimers.find(request.m_id);
+  if (r == m_activeTimers.end()) {
+    m_activeTimers.insert(ActiveTimerInfoMap::value_type(request.m_id, ActiveTimerInfo(request.m_timer, request.m_serialNumber)));
+  }
+  else {
+    r->second.m_serialNumber = request.m_serialNumber;
+    r->second.m_timer        = request.m_timer;
+  }
+  m_expiryList.insert(TimerExpiryInfo(request.m_id, request.m_absoluteTime, request.m_serialNumber));
+}
 
-  PWaitAndSignal l(timerListMutex);
 
-  PTRACE(5, "PTLib\tMONITOR:timers=" << activeTimers.size());
+void PTimerList::ProcessTimerQueue()
+{
+  m_queueMutex.Wait();
 
   // process the requests in the timer request queue
-  while (!requestQueue.empty()) {
-    queueMutex.Wait();
-    RequestType request = requestQueue.front();
-    requestQueue.pop();
-    queueMutex.Signal();
+  while (!m_requestQueue.empty()) {
 
-    TimerInfoMapType::iterator r = activeTimers.find(request.id);
-    switch (request.action) {
+    RequestType request(m_requestQueue.front());
+    m_requestQueue.pop();
+    m_queueMutex.Signal();
+
+    switch (request.m_action) {
       case PTimerList::RequestType::Start:
-        if (r == activeTimers.end()) 
-          activeTimers.insert(TimerInfoMapType::value_type(request.id, TimerInfoType(request.timer)));
+        AddActiveTimer(request);
         break;
       case PTimerList::RequestType::Stop:
-        if (r != activeTimers.end()) 
-          activeTimers.erase(r);
+        {
+          ActiveTimerInfoMap::iterator r = m_activeTimers.find(request.m_id);
+          if (r != m_activeTimers.end()) 
+            m_activeTimers.erase(r);
+        }
         break;
       default:
         PAssertAlways("unknown timer request code");
         break;
     }
-    if (request.sync != NULL)
-      request.sync->Signal();
+    if (request.m_sync != NULL)
+      request.m_sync->Signal();
+
+    m_queueMutex.Wait();
   }
 
-  // calculate interval since last processing, and update time of last processing to now
-  PTimeInterval now = PTimer::Tick();
-  PTimeInterval sampleTime;
-  if (lastSample == 0 || lastSample > now)
-    sampleTime = 0;
-  else {
-    sampleTime = now - lastSample;
-    if (now < lastSample)
-      sampleTime += PMaxTimeInterval;
-  }
-  lastSample = now;
+  m_queueMutex.Signal();
+}
 
-  // process the timers and find the minimum amount of time remaining
-  // also remove any timer labelled as removed
-  PTimeInterval minTimeLeft = PMaxTimeInterval;
-  {
-    TimerInfoMapType::iterator r = activeTimers.begin();
-    while (r != activeTimers.end()) {
-      PTimeInterval oldMinTimeLeft(minTimeLeft);
-      if (!r->second.removed) 
-        r->second.timer->Process(sampleTime, minTimeLeft);
-      if (!r->second.removed) 
-        ++r;
-      else {
-        if (r == activeTimers.begin()) {
-          activeTimers.erase(r);
-          r = activeTimers.begin();
-        }
+PTimeInterval PTimerList::Process()
+{
+  m_timerThread = PThread::Current();
+
+  PTRACE(6, "PTLib\tMONITOR: timers=" << m_activeTimers.size() << ", expiries=" << m_expiryList.size());
+
+  // process the timer queue
+  ProcessTimerQueue();
+
+  // process timers that have expired
+  PInt64 now = PTimer::Tick().GetMilliSeconds();
+  while ((m_expiryList.size() > 0) && (m_expiryList.begin()->m_expireTime <= now)) {
+    TimerExpiryInfo expiry = *m_expiryList.begin();
+    m_expiryList.erase(m_expiryList.begin());
+
+    ActiveTimerInfoMap::iterator t = m_activeTimers.find(expiry.m_timerId);
+    if (t != m_activeTimers.end()) {
+      ActiveTimerInfo & timer = t->second;
+      if (expiry.m_serialNumber == timer.m_serialNumber) {
+        timer.m_timer->Process(now);
+        if (timer.m_timer->m_state != PTimer::Stopped)
+          m_expiryList.insert(TimerExpiryInfo(expiry.m_timerId, now + timer.m_timer->m_resetTime.GetMilliSeconds(), timer.m_serialNumber));
         else
-        {
-          TimerInfoMapType::iterator s = r;
-          --s;
-          activeTimers.erase(r);
-          r = s;
-        }
-        minTimeLeft = oldMinTimeLeft;
+          m_activeTimers.erase(t);
       }
     }
   }
 
-  // add in an new timers created while timers were processed
-  while (!addQueue.empty()) {
-    queueMutex.Wait();
-    RequestType request = addQueue.front();
-    addQueue.pop();
-    queueMutex.Signal();
+  // process the timer queue again
+  ProcessTimerQueue();
 
-    activeTimers.insert(TimerInfoMapType::value_type(request.id, TimerInfoType(request.timer)));
-    request.timer->Process(0, minTimeLeft);
+  // use oldest timer to calculate minimum time left
+  PTimeInterval minTimeLeft;
+  if (m_expiryList.size() == 0) 
+    minTimeLeft = 1000;
+  else {
+    minTimeLeft = m_expiryList.begin()->m_expireTime - now;
+    if (minTimeLeft.GetMilliSeconds() < PTimer::Resolution())
+      minTimeLeft = PTimer::Resolution();
+    if (minTimeLeft < 25)
+      minTimeLeft = 25;
   }
 
   return minTimeLeft;
 }
 
+
 ///////////////////////////////////////////////////////////////////////////////
 // PArgList
 
@@ -1191,10 +1127,8 @@ PBoolean PArgList::Parse(const char * spec, PBoolean optionsBeforeParams)
       parameterIndex.SetSize(param+1);
       parameterIndex[param++] = arg;
     }
-    else if (optionsBeforeParams && parameterIndex.GetSize() > 0) {
-      m_argsParsed = arg;
+    else if (optionsBeforeParams && parameterIndex.GetSize() > 0)
       break;
-    }
     else if (argStr == "--") {
       if (optionsBeforeParams)
         hadMinusMinus = PTrue; // ALL remaining args are parameters not options
@@ -1214,6 +1148,9 @@ PBoolean PArgList::Parse(const char * spec, PBoolean optionsBeforeParams)
     arg++;
   }
 
+  if (optionsBeforeParams)
+    m_argsParsed = arg;
+
   return param > 0;
 }
 
@@ -1544,9 +1481,9 @@ PProcess::PProcess(const char * manuf, const char * name,
   , buildNumber(build)
   , maxHandles(INT_MAX)
   , m_library(library)
+  , m_shuttingDown(false)
 {
-  activeThreads.DisallowDeleteObjects();
-  activeThreads.SetAt(_hptr(GetCurrentThreadId()), this);
+  m_activeThreads[GetCurrentThreadId()] = this;
 
   PProcessInstance = this;
 
@@ -1566,37 +1503,6 @@ PProcess::PProcess(const char * manuf, const char * name,
   }
 #endif
 
-#if defined(P_MINGW) || defined(_WIN32) || defined(P_MACOSX)
-
-#  ifdef P_VIDEO
-     PWLibStupidLinkerHacks::loadFakeVideoStuff = 1;
-#    if defined(P_MINGW) || defined(_WIN32)
-#      ifdef P_VFW_CAPTURE
-         PWLibStupidLinkerHacks::loadVideoForWindowsStuff = 1;
-#      endif
-#      if defined(P_DIRECTSHOW)
-         PWLibStupidLinkerHacks::loadDirectShowStuff  =1;
-#      endif
-#    endif
-#    ifdef P_FFVDEV
-       PWLibStupidLinkerHacks::loadFFVideoFileStuff = 1;
-#    endif
-#    ifdef P_VIDFILE
-       PWLibStupidLinkerHacks::loadVideoFileStuff = 1;
-#    endif
-#  endif 
-
-#  ifdef P_AUDIO
-#    if defined(P_MINGW) || defined(_WIN32)
-       PWLibStupidLinkerHacks::loadWindowsMultimediaStuff = 1;
-#    endif
-#    ifdef P_MACOSX
-       PWLibStupidLinkerHacks::loadCoreAudioStuff = 1;
-#    endif
-#  endif
-
-#endif // defined(P_MINGW) || defined(_WIN32) || defined(P_MACOSX)
-
   if (productName.IsEmpty())
     productName = executableFile.GetTitle().ToLower();
 
@@ -1630,6 +1536,7 @@ PProcess::PProcess(const char * manuf, const char * name,
 
 void PProcess::PreShutdown()
 {
+  PProcessInstance->m_shuttingDown = true;
   PProcessStartupList & startups = GetPProcessStartupList();
   for (PProcessStartupList::iterator startup = startups.begin(); startup != startups.end(); ++startup)
     startup->second->OnShutdown();
@@ -1666,8 +1573,45 @@ void PProcess::OnThreadStart(PThread & /*thread*/)
 }
 
 
-void PProcess::OnThreadEnded(PThread & /*thread*/)
+static void OutputTime(ostream & strm, const char * name, const PTimeInterval & cpu, const PTimeInterval & real)
 {
+  strm << ", " << name << '=' << cpu << " (";
+
+  if (real == 0)
+    strm << '0';
+  else {
+    unsigned percent = (unsigned)((cpu.GetMilliSeconds()*1000)/real.GetMilliSeconds());
+    if (percent == 0)
+      strm << '0';
+    else
+      strm << (percent/10) << '.' << (percent%10);
+  }
+
+  strm << "%)";
+}
+
+
+ostream & operator<<(ostream & strm, const PThread::Times & times)
+{
+  strm << "real=" << scientific << times.m_real;
+  OutputTime(strm, "kernel", times.m_kernel, times.m_real);
+  OutputTime(strm, "user", times.m_user, times.m_real);
+  OutputTime(strm, "both", times.m_kernel + times.m_user, times.m_real);
+  return strm;
+}
+
+
+void PProcess::OnThreadEnded(PThread & PTRACE_PARAM(thread))
+{
+#if PTRACING
+  const int LogLevel = 3;
+  if (PTrace::CanTrace(LogLevel)) {
+    PThread::Times times;
+    if (thread.GetTimes(times)) {
+      PTRACE(LogLevel, "PTLib\tThread ended: name=\"" << thread.GetThreadName() << "\", " << times);
+    }
+  }
+#endif
 }
 
 
@@ -1724,7 +1668,12 @@ PString PProcess::GetVersion(PBoolean full) const
 
 PString PProcess::GetLibVersion()
 {
-  return psprintf("%u.%u%s%u", MAJOR_VERSION, MINOR_VERSION, VersionStatus[BUILD_TYPE], BUILD_NUMBER);
+  return psprintf("%u.%u%s%u (svn:%u)",
+                  MAJOR_VERSION,
+                  MINOR_VERSION,
+                  VersionStatus[BUILD_TYPE],
+                  BUILD_NUMBER,
+                  SVN_REVISION);
 }
 
 
@@ -1909,13 +1858,12 @@ PThread * PThread::Current()
 
   PProcess & process = PProcess::Current();
 
-  process.activeThreadMutex.Wait();
-  PThread * thread = process.activeThreads.GetAt(_hptr(GetCurrentThreadId()));
-  if (thread == NULL)
-    thread = new PExternalThread;
-  process.activeThreadMutex.Signal();
+  PWaitAndSignal mutex(process.m_activeThreadMutex);
+  PProcess::ThreadMap::iterator it = process.m_activeThreads.find(GetCurrentThreadId());
+  if (it != process.m_activeThreads.end())
+    return it->second;
 
-  return thread;
+  return new PExternalThread;
 }
 
 
@@ -2199,42 +2147,32 @@ PReadWriteMutex::~PReadWriteMutex()
      done by the user of the class too, but it is easier to fix here than
      there so practicality wins out!
    */
-  while (!nestedThreads.IsEmpty())
+  while (!m_nestedThreads.empty())
     PThread::Sleep(10);
 }
 
 
-PReadWriteMutex::Nest * PReadWriteMutex::GetNest() const
+PReadWriteMutex::Nest * PReadWriteMutex::GetNest()
 {
-  PWaitAndSignal mutex(nestingMutex);
-  return nestedThreads.GetAt(POrdinalKey(_hptr(PThread::GetCurrentThreadId())));
+  PWaitAndSignal mutex(m_nestingMutex);
+  NestMap::iterator it = m_nestedThreads.find(PThread::GetCurrentThreadId());
+  return it != m_nestedThreads.end() ? &it->second : NULL;
 }
 
 
 void PReadWriteMutex::EndNest()
 {
-  nestingMutex.Wait();
-  nestedThreads.RemoveAt(POrdinalKey(_hptr(PThread::GetCurrentThreadId())));
-  nestingMutex.Signal();
+  m_nestingMutex.Wait();
+  m_nestedThreads.erase(PThread::GetCurrentThreadId());
+  m_nestingMutex.Signal();
 }
 
 
 PReadWriteMutex::Nest & PReadWriteMutex::StartNest()
 {
-  POrdinalKey threadId = _hptr(PThread::GetCurrentThreadId());
-
-  nestingMutex.Wait();
-
-  Nest * nest = nestedThreads.GetAt(threadId);
-
-  if (nest == NULL) {
-    nest = new Nest;
-    nestedThreads.SetAt(threadId, nest);
-  }
-
-  nestingMutex.Signal();
-
-  return *nest;
+  PWaitAndSignal mutex(m_nestingMutex);
+  // The std::map will create the entry if it doesn't exist
+  return m_nestedThreads[PThread::GetCurrentThreadId()];
 }
 
 
