@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23827 $
+ * $Revision: 24753 $
  * $Author: rjongbloed $
- * $Date: 2009-12-01 16:39:07 -0600 (Tue, 01 Dec 2009) $
+ * $Date: 2010-09-27 21:11:47 -0500 (Mon, 27 Sep 2010) $
  */
 
 #include <ptlib/socket.h>
@@ -50,6 +50,8 @@
 #include <sys/sysctl.h>
 // going to need the main thread for adjusting relative priority
 static pthread_t baseThread;
+#elif defined(P_LINUX)
+#include <sys/syscall.h>
 #endif
 
 #ifdef P_HAS_SEMAPHORES_XPG6
@@ -172,30 +174,29 @@ void PHouseKeepingThread::Main()
 
     process.breakBlock.Wait(delay);
 
-    process.activeThreadMutex.Wait();
+    process.m_activeThreadMutex.Wait();
     PBoolean found;
     do {
-      found = PFalse;
-      for (PINDEX i = 0; i < process.activeThreads.GetSize(); ++i) {
-        PThread & thread = process.activeThreads.GetDataAt(i);
-        if (thread.autoDelete && thread.IsTerminated()) {
+      found = false;
+      for (PProcess::ThreadMap::iterator it = process.m_activeThreads.begin(); it != process.m_activeThreads.end(); ++it) {
+        PThread * thread = it->second;
+        if (thread->autoDelete && thread->IsTerminated()) {
+          process.m_activeThreads.erase(it);
 
-          // make sure the thread is removed from the list inside the mutex
-          // this means no other new thread can sneak in and reuse the ID 
-          PThread * toDelete = process.activeThreads.RemoveAt(process.activeThreads.GetKeyAt(i));
-
-          // unlock the activeThreadMutex to avoid deadlocks:
+          // unlock the m_activeThreadMutex to avoid deadlocks:
           // if somewhere in the destructor a call to PTRACE() is made,
           // which itself calls PThread::Current(), deadlocks are possible
-          process.activeThreadMutex.Signal();
-          delete toDelete;
-          process.activeThreadMutex.Wait();
-          found = PTrue;
+          thread->PX_threadId = 0;
+          process.m_activeThreadMutex.Signal();
+          delete thread;
+          process.m_activeThreadMutex.Wait();
+
+          found = true;
           break;
         }
       }
-    } while (found == PTrue); 
-    process.activeThreadMutex.Signal();
+    } while (found);
+    process.m_activeThreadMutex.Signal();
 
     process.PXCheckSignals();
   }
@@ -204,8 +205,11 @@ void PHouseKeepingThread::Main()
 }
 
 
-void PProcess::SignalTimerChange()
+bool PProcess::SignalTimerChange()
 {
+  if (m_shuttingDown)
+    return false;
+
   PWaitAndSignal m(housekeepingMutex);
   if (housekeepingThread == NULL) {
 #if PMEMORY_CHECK
@@ -218,6 +222,7 @@ void PProcess::SignalTimerChange()
   }
 
   breakBlock.Signal();
+  return true;
 }
 
 
@@ -292,20 +297,27 @@ PProcess::~PProcess()
 
 PBoolean PProcess::PThreadKill(pthread_t id, unsigned sig)
 {
-  PWaitAndSignal m(activeThreadMutex);
+  PWaitAndSignal m(m_activeThreadMutex);
 
-  if (!activeThreads.Contains(_hptr(id))) 
-    return PFalse;
+  if (m_activeThreads.find(id) == m_activeThreads.end()) 
+    return false;
 
   return pthread_kill(id, sig) == 0;
 }
 
 void PProcess::PXSetThread(pthread_t id, PThread * thread)
 {
-  activeThreadMutex.Wait();
-  PThread * currentThread = activeThreads.GetAt(_hptr(id));
-  activeThreads.SetAt(_hptr(id), thread);
-  activeThreadMutex.Signal();
+  PThread * currentThread = NULL;
+
+  m_activeThreadMutex.Wait();
+
+  ThreadMap::iterator it = m_activeThreads.find(id);
+  if (it != m_activeThreads.end() && it->second->autoDelete)
+    currentThread = it->second;
+
+  m_activeThreads[id] = thread;
+
+  m_activeThreadMutex.Signal();
 
   if (currentThread != NULL) 
     delete currentThread;
@@ -330,6 +342,9 @@ PThread::PThread()
   // PX_origStackSize = 0 indicates external thread
   PX_origStackSize    = 0;
   PX_threadId         = pthread_self();
+#if defined(P_LINUX)
+  PX_linuxId          = syscall(SYS_gettid);
+#endif
   PX_priority         = NormalPriority;
   PX_suspendCount     = 0;
 
@@ -384,6 +399,9 @@ PThread::PThread(PINDEX stackSize,
 
   // PX_threadId = 0 indicates thread has not started
   PX_threadId = 0;                          
+#if defined(P_LINUX)
+  PX_linuxId = 0;
+#endif
 
 #ifndef P_HAS_SEMAPHORES
   PX_waitingSemaphore = NULL;
@@ -435,11 +453,11 @@ PThread::~PThread()
 
     // if thread was started, remove it from the active thread list and detach it to release thread resources
     if (id != 0) {
-      process.activeThreadMutex.Wait();
+      process.m_activeThreadMutex.Wait();
       if (PX_origStackSize != 0)
         pthread_detach(id);
-      process.activeThreads.SetAt(_hptr(id), NULL);
-      process.activeThreadMutex.Signal();
+      process.m_activeThreads.erase(id);
+      process.m_activeThreadMutex.Signal();
     }
 
     // cause the housekeeping thread to wake up (we know it must be running)
@@ -469,6 +487,10 @@ void * PThread::PX_ThreadStart(void * arg)
   // been set.
   pthread_mutex_lock(&thread->PX_suspendMutex);
   thread->SetThreadName(thread->GetThreadName());
+#if defined(P_LINUX)
+  thread->PX_linuxId = syscall(SYS_gettid);
+  thread->PX_startTick = PTimer::Tick();
+#endif
   pthread_mutex_unlock(&thread->PX_suspendMutex);
 
   // make sure the cleanup routine is called when the thread exits
@@ -501,6 +523,11 @@ void PThread::PX_ThreadEnd(void * arg)
 {
   PThread * thread = (PThread *)arg;
   PProcess & process = PProcess::Current();
+
+#if defined(P_LINUX)
+  thread->PX_endTick = PTimer::Tick();
+#endif
+
   process.OnThreadEnded(*thread);
   
   bool deleteThread = thread->autoDelete; // make copy of the flag before perhaps deleting the thread
@@ -536,22 +563,22 @@ void PThread::Restart()
 #endif
 
   PProcess & process = PProcess::Current();
-  PINDEX newHighWaterMark = 0;
-  static PINDEX highWaterMark = 0;
+  size_t newHighWaterMark = 0;
+  static size_t highWaterMark = 0;
 
   // lock the thread list
-  process.activeThreadMutex.Wait();
+  process.m_activeThreadMutex.Wait();
 
   // create the thread
   PAssertPTHREAD(pthread_create, (&PX_threadId, &threadAttr, PX_ThreadStart, this));
 
   // put the thread into the thread list
   process.PXSetThread(PX_threadId, this);
-  if (process.activeThreads.GetSize() > highWaterMark)
-    newHighWaterMark = highWaterMark = process.activeThreads.GetSize();
+  if (process.m_activeThreads.size() > highWaterMark)
+    newHighWaterMark = highWaterMark = process.m_activeThreads.size();
 
   // unlock the thread list
-  process.activeThreadMutex.Signal();
+  process.m_activeThreadMutex.Signal();
 
   PTRACE_IF(4, newHighWaterMark > 0, "PTLib\tThread high water mark set: " << newHighWaterMark);
 
@@ -920,45 +947,88 @@ PBoolean PThread::IsTerminated() const
 //
 void PThread::WaitForTermination() const
 {
-  // if thread not started or is the current thread, then nothing to do
-  pthread_t id = PX_threadId;
-  if ((id == 0) || (id == Current()->GetThreadId())) {
-    PTRACE(2, "WaitForTermination short circuited");
-    return;
-  }
-  
-  // this assist in clean shutdowns on some systems
-  PXAbortBlock();   
-
-  // wait for termination
-  while (!IsTerminated()) {
-    Sleep(10); // sleep for 10ms. This slows down the busy loop removing 100%
-               // CPU usage and also yeilds so other threads can run.
-  }
-
-  PTRACE(5, "WaitForTermination on " << id << " finished");
+  WaitForTermination(PMaxTimeInterval);
 }
 
 
 PBoolean PThread::WaitForTermination(const PTimeInterval & maxWait) const
 {
-  if (this == Current()) {
-    PTRACE(2, "WaitForTermination(t) short circuited");
-    return PTrue;
+  pthread_t id = PX_threadId;
+  if ((id == 0) || (id == Current()->GetThreadId())) {
+    PTRACE(2, "WaitForTermination on 0x" << hex << id << dec << " short circuited");
+    return true;
   }
   
-  PTRACE(6, "PTLib\tWaitForTermination(" << maxWait << ')');
+  PTRACE(6, "WaitForTermination on 0x" << hex << id << dec << " for " << maxWait);
 
   PXAbortBlock();   // this assist in clean shutdowns on some systems
-  PTimer timeout = maxWait;
+
+  PTimeInterval startWaitTick = PTimer::Tick();
   while (!IsTerminated()) {
-    if (timeout == 0)
-      return PFalse;
+    if ((PTimer::Tick() - startWaitTick) > maxWait)
+      return false;
+
     Sleep(10); // sleep for 10ms. This slows down the busy loop removing 100%
                // CPU usage and also yeilds so other threads can run.
   }
 
-  return PTrue;
+  PTRACE(6, "WaitForTermination on 0x" << hex << id << dec << " finished");
+  return true;
+}
+
+
+static inline unsigned long long jiffies_to_msecs(const unsigned long j)
+{
+  static long sysconf_HZ = sysconf(_SC_CLK_TCK);
+  return (j * 1000LL) / sysconf_HZ;
+}
+
+
+bool PThread::GetTimes(Times & times)
+{
+#if defined(P_LINUX)
+  PStringStream procname;
+  procname << "/proc/" << getpid() << "/task/" << PX_linuxId << "/stat";
+  PTextFile statfile(procname, PFile::ReadOnly);
+  if (!statfile.IsOpen()) {
+    PTRACE(2, "PTLib\tCould not find thread stat file " << procname);
+    return false;
+  }
+
+  statfile.ignore(10000, ')'); // Skip pid & filename of executable
+
+  char state;
+  unsigned long dummy, utime, stime;
+  statfile >> state
+           >> dummy // ppid
+           >> dummy // pgrp
+           >> dummy // session
+           >> dummy // tty_nr
+           >> dummy // tpgid
+           >> dummy // flags
+           >> dummy // minflt
+           >> dummy // cminflt
+           >> dummy // majflt
+           >> dummy // cmajflt
+           >> utime
+           >> stime;
+  if (!statfile.good()) {
+    PTRACE(2, "PTLib\tCould not parse thread stat file " << procname);
+    return false;
+  }
+
+  times.m_kernel = jiffies_to_msecs(stime);
+  times.m_user = jiffies_to_msecs(utime);
+
+  if (PX_endTick != 0)
+    times.m_real = PX_endTick - PX_startTick;
+  else
+    times.m_real = PTimer::Tick() - PX_startTick;
+
+  return true;
+#else
+  return false;
+#endif
 }
 
 
@@ -1011,7 +1081,7 @@ int PThread::PXBlockOnIO(int handle, int type, const PTimeInterval & timeout)
 
   if ((retval == 1) && read_fds.IsPresent(unblockPipe[0])) {
     BYTE ch;
-    ::read(unblockPipe[0], &ch, 1);
+    PAssertOS(::read(unblockPipe[0], &ch, 1) != -1);
     errno = EINTR;
     retval =  -1;
     PTRACE(6, "PTLib\tUnblocked I/O fd=" << unblockPipe[0]);
@@ -1023,7 +1093,7 @@ int PThread::PXBlockOnIO(int handle, int type, const PTimeInterval & timeout)
 void PThread::PXAbortBlock() const
 {
   static BYTE ch = 0;
-  ::write(unblockPipe[1], &ch, 1);
+  PAssertOS(::write(unblockPipe[1], &ch, 1) != -1);
   PTRACE(6, "PTLib\tUnblocking I/O fd=" << unblockPipe[0] << " thread=" << GetThreadName());
 }
 
